package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.ExamTaskDetailDto;
import com.qmth.distributed.print.business.bean.params.RelatePaperParam;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.ExamDetailStatusEnum;
import com.qmth.distributed.print.business.enums.TaskTypeEnum;
import com.qmth.distributed.print.business.mapper.ExamTaskDetailMapper;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.templete.execute.AsyncCreatePdfTempleteService;
import com.qmth.distributed.print.business.util.ServletUtil;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * <p>
 * 命题任务明细表（入库申请） 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class ExamTaskDetailServiceImpl extends ServiceImpl<ExamTaskDetailMapper, ExamTaskDetail> implements ExamTaskDetailService {

    @Autowired
    private ExamTaskService examTaskService;

    @Autowired
    private ExamDetailService examDetailService;

    @Autowired
    private ExamDetailCourseService examDetailCourseService;

    @Autowired
    private ExamStudentService examStudentService;

    @Autowired
    private CommonService commonService;

    @Resource
    private AsyncCreatePdfTempleteService asyncCreatePdfTempleteService;

    @Autowired
    private BasicAttachmentService basicAttachmentService;

    @Resource
    TBTaskService tbTaskService;

    @Override
    public boolean enable(ExamTaskDetail examTaskDetail) {
        UpdateWrapper<ExamTaskDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTaskDetail::getEnable, examTaskDetail.getEnable()).eq(ExamTaskDetail::getExamTaskId, examTaskDetail.getId());
        return this.update(updateWrapper);
    }

    @Transactional
    @Override
    public boolean updatePaper(RelatePaperParam paperParam) throws IOException {
        ExamTask examTask = examTaskService.getById(paperParam.getExamTaskId());

        // 提交印刷的考场，不允许修改关联试卷
        List<ExamDetail> examDetails = examDetailService.listByCourseCodeAndPaperNumber(examTask.getSchoolId(), examTask.getCourseCode(), examTask.getPaperNumber());
        examDetails = examDetails.stream().filter(m -> !ExamDetailStatusEnum.NEW.equals(m.getStatus()) && !ExamDetailStatusEnum.READY.equals(m.getStatus())).collect(Collectors.toList());
        if (!examDetails.isEmpty()) {
            throw ExceptionResultEnum.ERROR.exception("有考场已开始打印，不能修改");
        }

        // 更新关联卷型
        ExamTask newExamTask = examTaskService.getByCourseCodeAndPaperNumber(examTask.getSchoolId(), examTask.getCourseCode(), paperParam.getPaperNumber());
        UpdateWrapper<ExamTaskDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTaskDetail::getRelatePaperType, paperParam.getRelatePaperType()).eq(ExamTaskDetail::getExamTaskId, newExamTask.getId());
        this.update(updateWrapper);

        // 更新试卷编号
        List<ExamDetailCourse> examDetailCourses = examDetailCourseService.listDetailCourseByCourseCodeAndPaperNumber(examTask.getSchoolId(), examTask.getCourseCode(), examTask.getPaperNumber());
        examDetailCourseService.updatePaperNumber(examDetailCourses, paperParam.getPaperNumber());

        // 更新考生关联类型
        examStudentService.updatePaperType(examDetailCourses, paperParam.getRelatePaperType());

        // 重新生成pdf
        List<Long> examDetailCourseIds = examDetailCourses.stream().map(m -> m.getId()).collect(Collectors.toList());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Map<String, Object> map = tbTaskService.saveTask(TaskTypeEnum.CREATE_PDF, paperParam.getPrintPlanId(), sysUser);
        map.computeIfAbsent("examDetailCourseIds", v -> examDetailCourseIds);
        map.computeIfAbsent("paperType", v -> paperParam.getRelatePaperType());
        asyncCreatePdfTempleteService.createPdf(map, null);
        return true;
    }

    @Override
    public IPage<ExamTaskDetailDto> list(String relateType, Long printPlanId, String courseCode, String paperNumber, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Set<Long> orgIds = commonService.listSubOrgIds(null);
        Page<ExamTaskDetailDto> page = new Page<>(pageNumber, pageSize);
        IPage<ExamTaskDetailDto> examDetailDtoIPage = this.baseMapper.listPage(page, schoolId, relateType, printPlanId, courseCode, paperNumber, orgIds);
        return examDetailDtoIPage;
    }

    @Override
    public List<ExamTaskDetail> listByCardId(Long cardId) {
        QueryWrapper<ExamTaskDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamTaskDetail::getCardId, cardId);
        return this.list(queryWrapper);
    }

    @Override
    public void bindCardId(String examTaskId, Long cardId) {
        UpdateWrapper<ExamTaskDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTaskDetail::getCardId, cardId).eq(ExamTaskDetail::getExamTaskId, examTaskId);
        this.update(updateWrapper);
    }

    @Override
    public void resetCardId(Long id) {
        UpdateWrapper<ExamTaskDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTaskDetail::getCardId, null).eq(ExamTaskDetail::getExamTaskId, id);
        this.update(updateWrapper);
    }

    @Override
    public List<ExamTaskDetail> listByTemplateId(Long templateId) {
        return this.baseMapper.listByTemplateId(templateId);
    }

    @Override
    public String getUrl(Long schoolId, Long examTaskId) {
        QueryWrapper<ExamTaskDetail> examTaskDetailQueryWrapper = new QueryWrapper<>();
        examTaskDetailQueryWrapper.lambda().eq(ExamTaskDetail::getExamTaskId, examTaskId);
        ExamTaskDetail examTaskDetail = this.getById(examTaskDetailQueryWrapper);
        if(examTaskDetail != null){
            String relatePaperType = examTaskDetail.getRelatePaperType();
            if(StringUtils.isNotBlank(relatePaperType)){
                String paperAttachmentIds = examTaskDetail.getPaperAttachmentIds();
                List<Map> list = JSONObject.parseArray(paperAttachmentIds, Map.class);
                String attachmentId = null;
                for (Map map : list) {
                    String name = map.get("name").toString();
                    if(relatePaperType.equals(name)){
                        attachmentId = map.get("attachmentId").toString();
                    }
                }
                if(StringUtils.isNotBlank(attachmentId)){
                    BasicAttachment attachment = basicAttachmentService.getById(attachmentId);
                    return attachment == null ? null : commonService.filePreview(attachment.getPath());
                }
            }
        }
        return null;
    }

}
